'use strict';












var _crypto = require('crypto');var _crypto2 = _interopRequireDefault(_crypto);
var _path = require('path');var _path2 = _interopRequireDefault(_path);
var _jestValidate = require('jest-validate');
var _chalk = require('chalk');var _chalk2 = _interopRequireDefault(_chalk);
var _glob = require('glob');var _glob2 = _interopRequireDefault(_glob);
var _jestResolve = require('jest-resolve');var _jestResolve2 = _interopRequireDefault(_jestResolve);
var _jestRegexUtil = require('jest-regex-util');var _jestRegexUtil2 = _interopRequireDefault(_jestRegexUtil);
var _utils = require('./utils');







var _constants = require('./constants');




var _reporter_validation_errors = require('./reporter_validation_errors');
var _defaults = require('./defaults');var _defaults2 = _interopRequireDefault(_defaults);
var _deprecated = require('./deprecated');var _deprecated2 = _interopRequireDefault(_deprecated);
var _set_from_argv = require('./set_from_argv');var _set_from_argv2 = _interopRequireDefault(_set_from_argv);
var _valid_config = require('./valid_config');var _valid_config2 = _interopRequireDefault(_valid_config);function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };} /**
                                                                                                                                                                                                        * Copyright (c) 2014-present, Facebook, Inc. All rights reserved.
                                                                                                                                                                                                        *
                                                                                                                                                                                                        * This source code is licensed under the BSD-style license found in the
                                                                                                                                                                                                        * LICENSE file in the root directory of this source tree. An additional grant
                                                                                                                                                                                                        * of patent rights can be found in the PATENTS file in the same directory.
                                                                                                                                                                                                        *
                                                                                                                                                                                                        * 
                                                                                                                                                                                                        */const ERROR = `${_utils.BULLET}Validation Error`;const JSON_EXTENSION = '.json';const PRESET_NAME = 'jest-preset' + JSON_EXTENSION;const createConfigError = message => new _jestValidate.ValidationError(ERROR, message, _utils.DOCUMENTATION_NOTE);const setupPreset = (
options,
optionsPreset) =>
{
  let preset;
  const presetPath = (0, _utils._replaceRootDirInPath)(options.rootDir, optionsPreset);
  const presetModule = _jestResolve2.default.findNodeModule(
  presetPath.endsWith(JSON_EXTENSION) ?
  presetPath :
  _path2.default.join(presetPath, PRESET_NAME),
  {
    basedir: options.rootDir });



  try {
    // $FlowFixMe
    preset = require(presetModule);
  } catch (error) {
    throw createConfigError(`  Preset ${_chalk2.default.bold(presetPath)} not found.`);
  }

  if (options.setupFiles) {
    options.setupFiles = (preset.setupFiles || []).concat(options.setupFiles);
  }
  if (options.modulePathIgnorePatterns && preset.modulePathIgnorePatterns) {
    options.modulePathIgnorePatterns = preset.modulePathIgnorePatterns.concat(
    options.modulePathIgnorePatterns);

  }
  if (options.moduleNameMapper && preset.moduleNameMapper) {
    options.moduleNameMapper = Object.assign(
    {},
    options.moduleNameMapper,
    preset.moduleNameMapper,
    options.moduleNameMapper);

  }

  // $FlowFixMe
  return Object.assign({}, preset, options);
};

const setupBabelJest = options => {
  const basedir = options.rootDir;
  const transform = options.transform;
  let babelJest;
  if (transform) {
    const customJSPattern = Object.keys(transform).find(pattern => {
      const regex = new RegExp(pattern);
      return regex.test('a.js') || regex.test('a.jsx');
    });

    if (customJSPattern) {
      const jsTransformer = _jestResolve2.default.findNodeModule(
      transform[customJSPattern],
      { basedir });

      if (
      jsTransformer &&
      jsTransformer.includes(_constants.NODE_MODULES + 'babel-jest'))
      {
        babelJest = jsTransformer;
      }
    }
  } else {
    babelJest = _jestResolve2.default.findNodeModule('babel-jest', { basedir });
    if (babelJest) {
      options.transform = {
        [_constants.DEFAULT_JS_PATTERN]: 'babel-jest' };

    }
  }

  return babelJest;
};

const normalizeCollectCoverageOnlyFrom = (
options,
key) =>
{
  const collectCoverageOnlyFrom = Array.isArray(options[key]) ?
  options[key] // passed from argv
  : Object.keys(options[key]); // passed from options
  return collectCoverageOnlyFrom.reduce((map, filePath) => {
    filePath = _path2.default.resolve(
    options.rootDir,
    (0, _utils._replaceRootDirInPath)(options.rootDir, filePath));

    map[filePath] = true;
    return map;
  }, Object.create(null));
};

const normalizeCollectCoverageFrom = (options, key) => {
  let value;
  if (!options[key]) {
    value = [];
  }

  if (!Array.isArray(options[key])) {
    try {
      value = JSON.parse(options[key]);
    } catch (e) {}

    Array.isArray(value) || (value = [options[key]]);
  } else {
    value = options[key];
  }

  return value;
};

const normalizeUnmockedModulePathPatterns = (
options,
key) =>
{
  // _replaceRootDirTags is specifically well-suited for substituting
  // <rootDir> in paths (it deals with properly interpreting relative path
  // separators, etc).
  //
  // For patterns, direct global substitution is far more ideal, so we
  // special case substitutions for patterns here.
  return options[key].map(pattern =>
  _jestRegexUtil2.default.replacePathSepForRegex(
  pattern.replace(/<rootDir>/g, options.rootDir)));


};

const normalizePreprocessor = options => {
  if (options.scriptPreprocessor && options.transform) {
    throw createConfigError(
    `  Options: ${_chalk2.default.bold('scriptPreprocessor')} and ${_chalk2.default.bold('transform')} cannot be used together.
  Please change your configuration to only use ${_chalk2.default.bold('transform')}.`);

  }

  if (options.preprocessorIgnorePatterns && options.transformIgnorePatterns) {
    throw createConfigError(
    `  Options ${_chalk2.default.bold('preprocessorIgnorePatterns')} and ${_chalk2.default.bold('transformIgnorePatterns')} cannot be used together.
  Please change your configuration to only use ${_chalk2.default.bold('transformIgnorePatterns')}.`);

  }

  if (options.scriptPreprocessor) {
    options.transform = {
      '.*': options.scriptPreprocessor };

  }

  if (options.preprocessorIgnorePatterns) {
    options.transformIgnorePatterns = options.preprocessorIgnorePatterns;
  }

  delete options.scriptPreprocessor;
  delete options.preprocessorIgnorePatterns;
  return options;
};

const normalizeMissingOptions = options => {
  if (!options.name) {
    options.name = _crypto2.default.
    createHash('md5').
    update(options.rootDir).
    digest('hex');
  }

  if (!options.setupFiles) {
    options.setupFiles = [];
  }

  return options;
};

const normalizeRootDir = options => {
  // Assert that there *is* a rootDir
  if (!options.hasOwnProperty('rootDir')) {
    throw createConfigError(
    `  Configuration option ${_chalk2.default.bold('rootDir')} must be specified.`);

  }
  options.rootDir = _path2.default.normalize(options.rootDir);
  return options;
};

const normalizeReporters = (options, basedir) => {
  const reporters = options.reporters;
  if (!reporters || !Array.isArray(reporters)) {
    return options;
  }

  (0, _reporter_validation_errors.validateReporters)(reporters);
  options.reporters = reporters.map(reporterConfig => {
    const normalizedReporterConfig = typeof reporterConfig ===
    'string' ?
    // if reporter config is a string, we wrap it in an array
    // and pass an empty object for options argument, to normalize
    // the shape.
    [reporterConfig, {}] :
    reporterConfig;

    const reporterPath = (0, _utils._replaceRootDirInPath)(
    options.rootDir,
    normalizedReporterConfig[0]);


    if (reporterPath !== _constants.DEFAULT_REPORTER_LABEL) {
      const reporter = _jestResolve2.default.findNodeModule(reporterPath, {
        basedir: options.rootDir });

      if (!reporter) {
        throw new Error(
        `Could not resolve a module for a custom reporter.\n` +
        `  Module name: ${reporterPath}`);

      }
      normalizedReporterConfig[0] = reporter;
    }
    return normalizedReporterConfig;
  });

  return options;
};

function normalize(options, argv) {var _validate =
  (0, _jestValidate.validate)(options, {
    comment: _utils.DOCUMENTATION_NOTE,
    deprecatedConfig: _deprecated2.default,
    exampleConfig: _valid_config2.default });const hasDeprecationWarnings = _validate.hasDeprecationWarnings;


  options = normalizePreprocessor(
  normalizeReporters(
  normalizeMissingOptions(normalizeRootDir((0, _set_from_argv2.default)(options, argv)))));



  if (options.preset) {
    options = setupPreset(options, options.preset);
  }

  if (options.testEnvironment) {
    options.testEnvironment = (0, _utils.getTestEnvironment)(options);
  }

  if (!options.roots && options.testPathDirs) {
    options.roots = options.testPathDirs;
    delete options.testPathDirs;
  }
  if (!options.roots) {
    options.roots = [options.rootDir];
  }

  if (!options.testRunner || options.testRunner === 'jasmine2') {
    options.testRunner = require.resolve('jest-jasmine2');
  }

  const babelJest = setupBabelJest(options);
  const newOptions = Object.assign({}, _defaults2.default);
  // Cast back to exact type
  options = options;
  Object.keys(options).reduce((newOptions, key) => {
    let value;
    switch (key) {
      case 'collectCoverageOnlyFrom':
        value = normalizeCollectCoverageOnlyFrom(options, key);
        break;
      case 'setupFiles':
      case 'snapshotSerializers':
        value =
        options[key] &&
        options[key].map(_utils.resolve.bind(null, options.rootDir, key));
        break;
      case 'modulePaths':
      case 'roots':
        value =
        options[key] &&
        options[key].map(filePath =>
        _path2.default.resolve(
        options.rootDir,
        (0, _utils._replaceRootDirInPath)(options.rootDir, filePath)));


        break;
      case 'collectCoverageFrom':
        value = normalizeCollectCoverageFrom(options, key);
        break;
      case 'cacheDirectory':
      case 'coverageDirectory':
        value =
        options[key] &&
        _path2.default.resolve(
        options.rootDir,
        (0, _utils._replaceRootDirInPath)(options.rootDir, options[key]));

        break;
      case 'moduleLoader':
      case 'resolver':
      case 'setupTestFrameworkScriptFile':
      case 'testResultsProcessor':
      case 'testRunner':
        value = options[key] && (0, _utils.resolve)(options.rootDir, key, options[key]);
        break;
      case 'moduleNameMapper':
        const moduleNameMapper = options[key];
        value =
        moduleNameMapper &&
        Object.keys(moduleNameMapper).map(regex => {
          const item = moduleNameMapper && moduleNameMapper[regex];
          return item && [regex, (0, _utils._replaceRootDirTags)(options.rootDir, item)];
        });
        break;
      case 'transform':
        const transform = options[key];
        value =
        transform &&
        Object.keys(transform).map(regex => [
        regex,
        (0, _utils.resolve)(options.rootDir, key, transform[regex])]);

        break;
      case 'coveragePathIgnorePatterns':
      case 'modulePathIgnorePatterns':
      case 'testPathIgnorePatterns':
      case 'transformIgnorePatterns':
      case 'unmockedModulePathPatterns':
        value = normalizeUnmockedModulePathPatterns(options, key);
        break;
      case 'haste':
        value = Object.assign({}, options[key]);
        if (value.hasteImplModulePath != null) {
          value.hasteImplModulePath = (0, _utils.resolve)(
          options.rootDir,
          'haste.hasteImplModulePath',
          (0, _utils._replaceRootDirInPath)(options.rootDir, value.hasteImplModulePath));

        }
        break;
      case 'projects':
        const projects = options[key];
        let list = [];
        projects &&
        projects.forEach(
        filePath =>
        list = list.concat(
        _glob2.default.sync((0, _utils._replaceRootDirInPath)(options.rootDir, filePath))));


        value = list;
        break;
      case 'moduleDirectories':
      case 'testMatch':
        value = (0, _utils._replaceRootDirTags)(options.rootDir, options[key]);
        break;
      case 'automock':
      case 'bail':
      case 'browser':
      case 'cache':
      case 'clearMocks':
      case 'collectCoverage':
      case 'coverageReporters':
      case 'coverageThreshold':
      case 'expand':
      case 'globals':
      case 'logHeapUsage':
      case 'mapCoverage':
      case 'moduleFileExtensions':
      case 'name':
      case 'noStackTrace':
      case 'notify':
      case 'replname':
      case 'reporters':
      case 'resetMocks':
      case 'resetModules':
      case 'rootDir':
      case 'silent':
      case 'testEnvironment':
      case 'testNamePattern':
      case 'testRegex':
      case 'testURL':
      case 'timers':
      case 'useStderr':
      case 'verbose':
      case 'watch':
      case 'watchman':
        value = options[key];
        break;}

    newOptions[key] = value;
    return newOptions;
  }, newOptions);

  newOptions.updateSnapshot = argv.ci && !argv.updateSnapshot ?
  'none' :
  argv.updateSnapshot ? 'all' : 'new';

  if (babelJest) {
    const regeneratorRuntimePath = _jestResolve2.default.findNodeModule(
    'regenerator-runtime/runtime',
    { basedir: options.rootDir });


    if (regeneratorRuntimePath) {
      newOptions.setupFiles.unshift(regeneratorRuntimePath);
    }
  }

  if (options.testRegex && options.testMatch) {
    throw createConfigError(
    `  Configuration options ${_chalk2.default.bold('testMatch')} and` +
    ` ${_chalk2.default.bold('testRegex')} cannot be used together.`);

  }

  if (options.testRegex && !options.testMatch) {
    // Prevent the default testMatch conflicting with any explicitly
    // configured `testRegex` value
    newOptions.testMatch = [];
  }

  // If argv.json is set, coverageReporters shouldn't print a text report.
  if (argv.json) {
    newOptions.coverageReporters = (newOptions.coverageReporters || []).
    filter(reporter => reporter !== 'text');
  }

  return {
    hasDeprecationWarnings,
    options: newOptions };

}

module.exports = normalize;